#include "../../include/error.h"
#include "../../include/pool.h"
#include "../../include/mptp.h"
#include "../../include/util.h"

#include <errno.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ini.h>
#include <unistd.h>

int lm_pool_info_handler(void *data, const char *_section, const char *_key, const char *_value) {
  char      *section = (char *)_section, *value = (char *)_value, *key = (char *)_key;
  lm_pool_t *pool = data;

  if (!eq(pool->name, section))
    return 0;

  if (eq(key, POOL_INFO_SIZE))
    pool->info.size = atol(value);
  else if (eq(key, POOL_INFO_MAINTAINER))
    pool->info.maintainer = strdup(value);
  else if (eq(key, POOL_INFO_PUBKEY))
    pool->info.pubkey = strdup(value);
  else
    return 0;

  return 1;
}

bool lm_pool_info_load(lm_pool_t *pool, char *file) {
  lm_pool_info_free(pool);

  if(!can_read(file)){
    lm_error_set(LM_ERR_PoolInfoCantRead);
    return false;
  }

  if (ini_parse(file, lm_pool_info_handler, pool) < 0) {
    lm_error_set(LM_ERR_PoolInfoBad);
    return false;
  }

  pool->info.file = strdup(file);
  return true;
}

bool lm_pool_info_get(lm_pool_t *pool, char *file) {
  if(!pool->available)
    return false;

  if(NULL == pool->url.host)
    return false;

  if(NULL == pool->url.path)
    return false;

  FILE *info = fopen(file, "a");
  int sock = lm_mptp_client_connect(pool->url.host, pool->url.port);
  lm_mptp_t packet;
  bool ret = false;

  if(NULL == info)
    goto end;

  lm_mptp_init(&packet, true, MPTP_C2S_INFO, true);
  lm_mptp_set_host(&packet, pool->url.host);
  lm_mptp_set_data(&packet, pool->url.path, strlen(pool->url.path));

  if(!lm_mptp_client_send(sock, &packet))
    goto end;

  while(lm_mptp_client_recv(sock, &packet)){
    if(!lm_mptp_client_verify(&packet))
      goto end;
  
    if(MPTP_IS_LAST(&packet))
      break;

    if(fwrite(packet.data, 1, packet.header.data_size, info)==0)
      goto end;
  }

  ret = true;
end:
  fclose(info);
  if(ret)
    ret = lm_pool_info_load(pool, file);
  return ret;
}

void lm_pool_info_free(lm_pool_t *pool) {
  free(pool->info.file);
  free(pool->info.pubkey);
  free(pool->info.maintainer);
  bzero(&pool->info, sizeof(pool->info));
}
