#include "../../include/error.h"
#include "../../include/pool.h"
#include "../../include/mptp.h"
#include "../../include/util.h"
#include "../../include/pkg.h"

#include <dirent.h>
#include <libgen.h>
#include <stdbool.h>
#include <string.h>

bool lm_pool_list_load(lm_pool_t *pool, char *file){
  if(NULL == pool || NULL == file){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }

  if(!can_read(file)){
    lm_error_set(LM_ERR_PoolListCantRead);
    return false;
  }

  size_t file_len = strlen(file), ent_len = 0;
  char filecp[file_len+1], *dir = NULL;
  struct dirent *ent = NULL;
  DIR *dirfd = NULL;
  bool ret = false;

  memcpy(filecp, file, file_len+1);
  dir = dirname(filecp);

  if(!extract_archive(dir, file))
    goto end;

  if((dirfd = opendir(dir)) == NULL){
    lm_error_set(LM_ERR_PoolListDirFail);
    goto end;
  }
  
  while((ent = readdir(dirfd)) != NULL){
    ent_len = strlen(ent->d_name);

    char datap[ent_len+file_len+10];
    snprintf(datap, sizeof(datap), "%s/%s/DATA", dir, ent->d_name);

    lm_pkg_t *pkg = lm_pkg_new();
    
    if(!lm_pkg_data_load(pkg, datap)){
      lm_pkg_free(pkg);
      continue;
    }

    if(!lm_pool_add(pool, pkg)){
      lm_pkg_free(pkg);
      continue;
    }
  }

end:
  if(NULL != dirfd)
    closedir(dirfd);
  return ret;
}

bool lm_pool_list_get(lm_pool_t *pool, char *file) {
  if(!pool->available)
    return false;

  if(NULL == pool->url.host)
    return false;

  if(NULL == pool->url.path)
    return false;

  FILE *info = fopen(file, "a");
  int sock = lm_mptp_client_connect(pool->url.host, pool->url.port);
  lm_mptp_t packet;
  bool ret = false;

  if(NULL == info)
    goto end;

  lm_mptp_init(&packet, true, MPTP_C2S_LIST, true);
  lm_mptp_set_host(&packet, pool->url.host);
  lm_mptp_set_data(&packet, pool->url.path, strlen(pool->url.path));

  if(!lm_mptp_client_send(sock, &packet))
    goto end;

  if(!lm_mptp_client_recv(sock, &packet))
    goto end;

  while(lm_mptp_client_recv(sock, &packet)){
    if(!lm_mptp_client_verify(&packet))
      goto end;

    if(fwrite(packet.data, 1, packet.header.data_size, info)==0)
      goto end;

    if(MPTP_IS_LAST(&packet))
      break;
  }

  ret = true;
end:
  fclose(info);
  if(ret)
    ret = lm_pool_list_load(pool, file);
  return ret;
}
